#!/usr/bin/env python

from distutils.core import setup

setup(name="cycle",
      version="0.3.1",
      description="Calendar for women",
      author="Oleg Gints",
      author_email="altgo@users.sourceforge.net",
      url="http://cycle.sourceforge.net",
      py_modules=['cycle'],
      )
